/*  _ __ ___ _
 * | |\ /  /| |  $Id: CdlArg.C,v 1.5 1997/04/08 01:51:16 dpwe Exp $
 * | | /  / | |  Copyright (C) 1995 IXI Limited.
 * |_|/__/_\|_|  IXI Limited, Cambridge, England.
 *
 * Component   : CdlArg.C
 *
 * Author      : Dean Sheehan (deans@x.co.uk)
 *
 * Description : Contains the implementation of the CdlArg class that models
 *               arguments on methods in cdl.
 *
 * License     :
			Object Tcl License & Copyright
			------------------------------

IXI Object Tcl software, both binary and source (hereafter, Software) is copyrighted by IXI Limited (IXI), and ownership remains with IXI. 

IXI grants you (herafter, Licensee) a license to use the Software for academic, research and internal business purposes only, without a fee. Licensee may distribute the binary and source code (if required) to third parties provided that the copyright notice and this statement appears on all copies and that no charge is associated with such copies. 

Licensee may make derivative works. However, if Licensee distributes any derivative work based on or derived from the Software, then Licensee will (1) notify IXI regarding its distribution of the derivative work, and (2) clearly notify users that such derivative work is a modified version and not the original IXI Object Tcl distributed by IXI. IXI strongly recommends that Licensee provide IXI the right to incorporate such modifications into future releases of the Software under these license terms. 

Any Licensee wishing to make commercial use of the Software should contact IXI, to negotiate an appropriate license for such commercial use. Commercial use includes (1) integration of all or part of the source code into a product for sale or license by or on behalf of Licensee to third parties, or (2) distribution of the binary code or source code to third parties that need it to utilize a commercial product sold or licensed by or on behalf of Licensee. 

IXI MAKES NO REPRESENTATIONS ABOUT THE SUITABILITY OF THIS SOFTWARE FOR ANY PURPOSE. IT IS PROVIDED "AS IS" WITHOUT EXPRESS OR IMPLIED WARRANTY. IXI SHALL NOT BE LIABLE FOR ANY DAMAGES WHATSOEVER SUFFERED BY THE USERS OF THIS SOFTWARE. 

Copyright (C) 1995, IXI Limited 

By using or copying this Software, Licensee agrees to abide by the copyright law and all other applicable laws of England and the U.S., including, but not limited to, export control laws, and the terms of this license. IXI shall have the right to terminate this license immediately by written notice upon Licensee's breach of, or non-compliance with, any of its terms. Licensee may be held legally responsible for any copyright infringement that is caused or encouraged by Licensee's failure to abide by the terms of this license. 

Comments and questions are welcome and can be sent to
otcl@x.co.uk 

For more information on copyright and licensing issues, contact: 
Legal Department, IXI Limited, Vision Park, Cambridge CB4 4ZR,
ENGLAND. 

 *
 */

// System Includes
#include <string.h>
#include <stdlib.h>

// Local Includes
#include "CdlArg.H"
#include "CdlFile.H"

CdlArg *CdlArg::exemplarHead = NULL;
CdlArg *CdlArg::exemplarTail = NULL;

CdlArg::CdlArg (int e)
{
    defaultFlag = ARG_DEFAULT_NONE;	// no default val
    defaultVal = (char *)0;

   nextExemplar = NULL;
   if (e == 1)
   {
     // This is an exemplar
     if (exemplarTail == NULL)
     {
        exemplarHead = exemplarTail = this;
     }
     else
     {
        exemplarTail->nextExemplar = this;
        exemplarTail = this;
     }
   }
}

CdlArg::~CdlArg ()
{
}

int CdlArg::setArgs (int argc, char *argv[])
{
    // Used to just "return 1"; now checks for -default, -from
    if (argc == 0) {  // no extra args - OK
	return 1;
    } else if (argc == 2) {	// pair of extra args ...
	if (strcmp(argv[0],"-default") == 0) {
	    // store the default value of the optional arg
	    defaultVal = strdup(argv[1]);
	    defaultFlag = ARG_DEFAULT_VAL;
	    return 1;
	} else if (strcmp(argv[0],"-from") == 0) {
	    // store the name of the (dummy) slot from which to take the arg
	    defaultVal = strdup(argv[1]);
	    defaultFlag = ARG_DEFAULT_SLOT;
	    return 1;
	}
    }
    return 0;
}

ofstream &CdlArg::initFromVar (char *varName, char *fromName, ofstream &of,
				  CdlIndent tab, int rtnErr, int omitType)
{
    of << tab;
    if (!omitType) {
	type(of);
	of << " ";
    }
    of << varName << " = " << fromName << ";" << endl;
    return of;
}

// default cleanVar does nothing
ofstream &CdlArg::cleanVarFromStr (char *, char *,
                                     ofstream &of, CdlIndent)
{
   return of;
}

// default disposeVar does nothing
ofstream &CdlArg::disposeVar (char *, 
			      ofstream &of, CdlIndent)
{
   return of;
}


CDL_ARG_DEF(CdlIntArg,int)

CdlIntArg::~CdlIntArg ()
{
}

ofstream &CdlIntArg::type (ofstream &of)
{
   of << "int";
   return of;
}

ofstream &CdlIntArg::initVarFromStr (char *varName, char *strName, ofstream &of,
                                     CdlIndent tab, int rtnErr, int omitType, int firstTime)
{
    of << tab;
    if (!omitType) {
	of << "int ";
    }
    of << varName << " = atoi(" << strName << ");" << endl;
    return of;
}

ofstream &CdlIntArg::initStrFromVar (char *strName, char *varName,
                                     ofstream &of, CdlIndent tab)
{
   of << tab << strName << " = (char*)malloc(" << SYMBOLIC_REFERENCE_SIZE << "*sizeof(char));" << endl;
   setStrFromVar(strName, varName, of, tab);
   return of;
}

ofstream &CdlIntArg::setStrFromVar (char *strName, char *varName,
				    ofstream &of, CdlIndent tab)
{
   of << tab << "sprintf(" << strName << ",\"%d\"," << varName << ");" << endl;
   return of;
}

ofstream &CdlIntArg::cleanStrFromVar (char *strName, char *,
                                      ofstream &of, CdlIndent tab)
{
    of << tab << "free((char*)" << strName << ");" << endl;
   return of;
}


CDL_ARG_DEF(CdlFloatArg,float)

CdlFloatArg::~CdlFloatArg ()
{
}

ofstream &CdlFloatArg::type (ofstream &of)
{
   of << "float";
   return of;
}

ofstream &CdlFloatArg::initVarFromStr (char *varName, char *strName,
                                       ofstream &of,
                                       CdlIndent tab, int rtnErr, int omitType, int firstTime)
{
   of << tab;
   if (!omitType) {
       of << "float ";
   }  
   of << varName << " = atof(" << strName << ");" << endl;
   return of;
}

ofstream &CdlFloatArg::initStrFromVar (char *strName, char *varName,
                                     ofstream &of, CdlIndent tab)
{
   of << tab << strName << " = (char*)malloc(" << SYMBOLIC_REFERENCE_SIZE << "*sizeof(char));" << endl;
   setStrFromVar(strName, varName, of, tab);
   return of;
}

ofstream &CdlFloatArg::setStrFromVar (char *strName, char *varName,
                                     ofstream &of, CdlIndent tab)
{
   of << tab << "sprintf(" << strName << ",\"%f\"," << varName << ");" << endl;
   return of;
}

ofstream &CdlFloatArg::cleanStrFromVar (char *strName, char *,
                                      ofstream &of, CdlIndent tab)
{
    of << tab << "free((char*)" << strName << ");" << endl;
   return of;
}

CDL_ARG_DEF(CdlDoubleArg,double)

CdlDoubleArg::~CdlDoubleArg ()
{
}

ofstream &CdlDoubleArg::type (ofstream &of)
{
   of << "double";
   return of;
}

ofstream &CdlDoubleArg::initVarFromStr (char *varName, char *strName,
                                       ofstream &of,
                                       CdlIndent tab, int rtnErr, int omitType, int firstTime)
{
   of << tab;
   if(!omitType) {
       of << "double ";
   }
   of << varName << " = strtod(" << strName << ",(char**)NULL);" << endl;
   return of;
}

ofstream &CdlDoubleArg::initStrFromVar (char *strName, char *varName,
                                     ofstream &of, CdlIndent tab)
{
   of << tab << strName << " = (char*)malloc((" << SYMBOLIC_REFERENCE_SIZE << "*sizeof(char));" << endl;
   setStrFromVar(strName, varName, of, tab);
   return of;
}

ofstream &CdlDoubleArg::setStrFromVar (char *strName, char *varName,
                                     ofstream &of, CdlIndent tab)
{
   of << tab << "sprintf(" << strName << ",\"%f\"," << varName << ");" << endl;
   return of;
}

ofstream &CdlDoubleArg::cleanStrFromVar (char *strName, char *,
                                      ofstream &of, CdlIndent tab)
{
    of << tab << "free((char*)" << strName << ");" << endl;
   return of;
}

CDL_ARG_DEF(CdlStrArg,str)

CdlStrArg::~CdlStrArg ()
{
    shouldNew = CDL_FALSE;	// reminder that this exists
}

ofstream &CdlStrArg::type (ofstream &of)
{
   of << "char *";
   return of;
}

#define MAX_ARGC 32

int CdlStrArg::setArgs (int argc, char *argv[])
{   // Check for our own "-new" flag; pass rest to base class's version
    int i;
    int myargc = 0;
    char *myargv[MAX_ARGC];

    shouldNew = CDL_FALSE;	
    for(i = 0; i < argc; ++i) {
	if(strcmp(argv[i],"-new") == 0) {
	    shouldNew = CDL_TRUE;
	} else {
	    // copy unrecognize args to pass up
	    myargv[myargc++] = argv[i];
	}
    }
    return CdlArg::setArgs(myargc, myargv);
}

ofstream &CdlStrArg::initVarFromStr (char *varName, char *strName, ofstream &of,
                                     CdlIndent tab, int rtnErr, int omitType, int firstTime /* = 0 */)
{
    if (!firstTime && omitType) {
	disposeVar(varName, of, tab);
    }
    of << tab;
    if (!omitType) {
	of << "char *";
    }
    if (shouldNew == CDL_TRUE)  {
	of << varName << " = strdup(" << strName << ");" << endl;
    } else {
	of << varName << " = " << strName << ";" << endl;
    }
    return of;
}

ofstream &CdlStrArg::disposeVar (char *varName, 
                                     ofstream &of, CdlIndent tab)
{
    if(shouldNew == CDL_TRUE) {
	of << tab << "free((char *)" << varName << ");" << endl;
    }
    return of;
}

ofstream &CdlStrArg::initStrFromVar (char *strName, char *varName,
                                     ofstream &of, CdlIndent tab)
{
   of << tab << strName << " = strdup(" << varName << ");" << endl;
   return of;
}

ofstream &CdlStrArg::setStrFromVar (char *strName, char *varName,
                                     ofstream &of, CdlIndent tab)
{
    of << tab << "strcpy(" << strName << ", " << varName << ");" << endl;
   return of;
}

ofstream &CdlStrArg::cleanStrFromVar (char *strName, char *,
                                      ofstream &of, CdlIndent tab)
{
    of << tab << "free((char*)" << strName << ");" << endl;
   return of;
}

#include "CdlItem.H"	// for class ptr access

CDL_ARG_DEF(CdlObptrArg,obptr)

int CdlObptrArg::setArgs (int argc, char *argv[])
{
   if (argc < 1)
   {
      return 0;
   }
   className = strdup(argv[0]);
   classCName = className;
   return CdlArg::setArgs(argc-1, argv+1);
}

int CdlObptrArg::ensureCName(void) 
{
    CdlClass *classPtr;

    if(className == classCName) {
	if( (classPtr = CdlClass::findClass(className)) == NULL) {
	    fprintf(stderr, "error: CdlObptr: class '%s' not defined\n", 
		    className);
	    return 0;
	}
	classCName = strdup(classPtr->getCname());
//	fprintf(stderr, "CdlObptrArg: cname of %s is %s\n", 
//		className, classCName);
    }
    return 1;
}

CdlObptrArg::~CdlObptrArg ()
{
    if (classCName != className) {
	free(classCName);
    }
    if (className) {
	free(className);
    }
}

ofstream &CdlObptrArg::type (ofstream &of)
{
    ensureCName();
    of << classCName << " *";
    return of;
}

ofstream &CdlObptrArg::initVarFromStr (char *varName, char *strName,
                                       ofstream &of,
                                       CdlIndent tab, int rtnErr, int omitType, int firstTime)
{
    ensureCName();

   of << tab;
   if (!omitType) {
       of << classCName << " *";
   }
   of << varName << " = (" << classCName << "*)"
      << "Otcl::otclPtr->obrefToCpp("
      << strName << ",\"" << className << "\");" << endl;
   if(rtnErr) {
       // Test for null return and generate error if so
       of << tab << "if(" << varName << " == (" << classCName << "*)0) {" << endl; tab++;
       of << tab << "Tcl_AppendResult(i, \"could not convert \", " << strName << ", \" to an object of class " << className << "\", 0);" << endl;
       of << tab << "return TCL_ERROR;" << endl;
       tab--;
       of << "}" << endl;
   }

   return of;
}

ofstream &CdlObptrArg::initStrFromVar (char *strName, char *varName,
                                       ofstream &of, CdlIndent tab)
{
   of << tab << strName << " = (char*)malloc((" << SYMBOLIC_REFERENCE_SIZE << "*sizeof(char));" << endl;
   setStrFromVar(strName, varName, of, tab);
   return of;
}

ofstream &CdlObptrArg::setStrFromVar (char *strName, char *varName,
                                       ofstream &of, CdlIndent tab)
{
   of << tab << "strcpy(" << strName << "," 
      << "((" << className << OTCL_PART_SUFFIX << "*)" 
      << className << OTCL_CLASS_SUFFIX << "::pinstance->find_otcl(" 
      << varName << "))"
      << "->giveOwner()->getSelf()"
      << ");" << endl;
   return of;
}

ofstream &CdlObptrArg::cleanStrFromVar (char *strName, char *,
                                        ofstream &of, CdlIndent tab)
{
   of << tab << "free((char*)" << strName << ");" << endl;
   return of;
}

CDL_ARG_DEF(CdlObrefArg,obref)

int CdlObrefArg::setArgs (int argc, char *argv[])
{
   if (argc < 1) {
      return 0;
   }
   className = strdup(argv[0]);
   classCName = className;
   return CdlArg::setArgs(argc-1, argv+1);
}

int CdlObrefArg::ensureCName(void) 
{
    CdlClass *classPtr;

    if(className == classCName) {
	if( (classPtr = CdlClass::findClass(className)) == NULL) {
	    fprintf(stderr, "error: CdlObref: class '%s' not defined\n", 
		    className);
	    return 0;
	}
	classCName = strdup(classPtr->getCname());
//	fprintf(stderr, "CdlObptrArg: cname of %s is %s\n", 
//		className, classCName);
    }
    return 1;
}

CdlObrefArg::~CdlObrefArg ()
{
    if (classCName != className) {
	free(classCName);
    }
    if (className) {
	free(className);
    }
}

// unique to obref - allow inspection of the class name, 
// for detecting special-case copy constructor definition
char *CdlObrefArg::giveClass(void)  {
    return className;
}

ofstream &CdlObrefArg::type (ofstream &of)
{
    ensureCName();
    of << classCName << " &";
    return of;
}

ofstream &CdlObrefArg::initVarFromStr (char *varName, char *strName,
                                       ofstream &of,
                                       CdlIndent tab, int rtnErr, int omitType, int firstTime)
{
    ensureCName();

   of << tab;
   if (!omitType) {
       of << classCName << " &";
   }
   of << varName << " = *(" << classCName << "*)"
      << "Otcl::otclPtr->obrefToCpp("
      << strName << ",\"" << className << "\");" << endl;
   if(rtnErr) {
       // Test for null return and generate error if so
       of << tab << "if(&" << varName << " == (" << classCName << "*)0) {" << endl; tab++;
       of << tab << "Tcl_AppendResult(i, \"could not convert \", " << strName << ", \" to an object of class " << className << "\", 0);" << endl;
       of << tab << "return TCL_ERROR;" << endl;
       tab--;
       of << "}" << endl;
   }

   return of;
}

ofstream &CdlObrefArg::initStrFromVar (char *strName, char *varName,
                                       ofstream &of, CdlIndent tab)
{
   of << tab << strName << " = (char*)malloc((" << SYMBOLIC_REFERENCE_SIZE << "*sizeof(char));" << endl;
   setStrFromVar(strName, varName, of, tab);
   return of;
}

ofstream &CdlObrefArg::setStrFromVar (char *strName, char *varName,
                                       ofstream &of, CdlIndent tab)
{
   // invoke the new _otclc::find_otcl() method to convert the c++ pointer
   // into an _otcl pointer
   of << tab << "strcpy(" << strName << "," 
      << "((" << className << OTCL_PART_SUFFIX << "*)" 
      << className << OTCL_CLASS_SUFFIX << "::pinstance->find_otcl(&"
      << varName << "))"
      << "->giveOwner()->getSelf()"
      << ");" << endl;
   return of;
}

ofstream &CdlObrefArg::cleanStrFromVar (char *strName, char *,
                                        ofstream &of, CdlIndent tab)
{
   of << tab << "free((char*)" << strName << ");" << endl;
   return of;
}
