//
// CdlSlot.C
//
// Extension to Dean Sheenan's ObjectTcl 1.1
// to add a 'slot' command to the class definitions
// to generate automatic accessor functions to external C++ classes
//
// dpwe@media.mit.edu 1995nov11
//

// System Includes
#include <string.h>
#include <stdlib.h>	// for free()

// Local Includes
#include "CdlFile.H"
#include "CdlSlot.H"
#include "CdlRtn.H"
#include "CdlArg.H"	// used for setting slot in config

CdlSlot::CdlSlot(char *n)  : CdlMethod(n)
{
    setScript = NULL;
    getScript = NULL;
}

CdlSlot::~CdlSlot(void)
{
    if(setScript) 	free(setScript);
    if(getScript)	free(getScript);
}

void CdlSlot::setSetScript(char *str) {
    setScript = strdup(str);
}

void CdlSlot::setGetScript(char *str) {
    getScript = strdup(str);
}

void CdlSlot::executeBinding(char *className, ofstream &of)
{
    CdlIndent tab(1);

    // Generate a give_<slotname> method name
    char giveName[32];

    sprintf(giveName, "give_%s", name);
    of << tab << "if (strcmp(mName,\"" << giveName << "\") == 0)" << endl;
    of << tab << "{" << endl;
    tab++;

    of << tab << "*found = OTCL_TRUE;" << endl;

    of << tab << "if (argc != 0)" << endl;
    of << tab << "{" << endl;
    tab++;
    of << tab << "return Otcl::setTclError(i,ARGS_METHOD_ERR,"
	<< "\"" << giveName << "\",\"" << className << "\");" << endl;
    tab--;
    of << tab << "}" << endl;

    rtn->declareLocal(of,tab);
    of << "objPtr->" << name << ");" << endl;
    rtn->postProcess(of,tab);

    of << tab << "return TCL_OK;" << endl; tab--; of << tab << "}" <<
    endl;
}

void CdlSlot::setReturn(ofstream& of, CdlIndent& tab)
{
    rtn->declareLocal(of,tab);
    of << "objPtr->" << name << ");" << endl;
    rtn->postProcess(of,tab);
    
}

void CdlSlot::setValFromStr(char *strName, ofstream& of, CdlIndent& tab)
{
    char str[32];

    sprintf(str, "objPtr->%s", name);
    of << tab << "if(objPtr) {" << endl; tab++;
    arg[0]->initVarFromStr(str, strName, of, tab, 1, 1);
    tab--; of << tab << "}" << endl;
}

CdlFakeSlot::CdlFakeSlot(char *n)  : CdlSlot(n)
{
}

CdlFakeSlot::~CdlFakeSlot(void)
{
}

void CdlFakeSlot::setReturn(ofstream& of, CdlIndent& tab)
{
    rtn->declareLocal(of,tab);
    of << "this->" << name << ");" << endl;
    rtn->postProcess(of,tab);
}

void CdlFakeSlot::setValFromStr(char *strName, ofstream& of, CdlIndent& tab)
{
    char str[32];
    sprintf(str, "this->%s", name);
    arg[0]->initVarFromStr(str, strName, of, tab, 1, 1);
}

void CdlFakeSlot::genDec(ofstream& of) {
    CdlIndent tab(1);
    of << tab;
    rtn->genPrototype(of);
    of << " " << name << ";" << endl;
}

void CdlFakeSlot::genDefaultInit(ofstream& of) {
    CdlIndent tab(1);

    if (arg[0]->defaultFlag == ARG_DEFAULT_VAL) {
	char strName[32];
	sprintf(strName,"\"%s\"", arg[0]->defaultVal);
	//setValFromStr(strName, of, tab);
	// special case - DON'T want to free previous contents
	char slotName[32];
	sprintf(slotName, "this->%s", name);
	// call initVfrS with "firstTime" set, prevents a free()
	arg[0]->initVarFromStr(slotName, strName, of, tab, 1, 1, 1);
    }
}

void CdlFakeSlot::genDestruct(ofstream& of) {
    // put stuff in the destructor to clean up any allocation of 
    // fake slots (e.g. strings)
    CdlIndent tab(1);
    char slotName[32];
    sprintf(slotName, "this->%s", name);
    arg[0]->disposeVar(slotName, of, tab);
}

void CdlFakeSlot::setString(char *sname, ofstream& of, CdlIndent& tab)
{
    char varName[32];
    sprintf(varName, "this->%s", name);
    arg[0]->setStrFromVar(sname, varName, of, tab);
}

void CdlFakeSlot::initString(char *sname, ofstream &of, CdlIndent& tab)
{
    char varName[32];
    sprintf(varName, "this->%s", name);
    arg[0]->initStrFromVar(sname, varName, of, tab);
}

void CdlFakeSlot::cleanString(char *sname, ofstream &of, CdlIndent& tab)
{
    char varName[32];
    sprintf(varName, "this->%s", name);
    arg[0]->cleanStrFromVar(sname, varName, of, tab);
}

int CdlFakeSlot::isOK(Tcl_Interp* i)
{   // Check that whole slot is OK, which means that it has a default
    if(arg[0]->defaultFlag == ARG_DEFAULT_VAL) {
	return 1;
    } else {
	Tcl_SetResult(i, "dummy slots must have -default values", TCL_STATIC);
	return 0;
    }
}

CdlClassSlot::CdlClassSlot(char *n)  : CdlSlot(n)
{
}

CdlClassSlot::~CdlClassSlot(void)
{
}

void CdlClassSlot::setReturn(char *className, ofstream& of, CdlIndent& tab)
{
    rtn->declareLocal(of,tab);
    of << className << "::" << name << ");" << endl;
    rtn->postProcess(of,tab);
    
}

void CdlClassSlot::setValFromStr(char *className, char *strName, ofstream& of, CdlIndent& tab)
{
    arg[0]->initVarFromStr(name, strName, of, tab, 1);
    of << tab << className << "::" << name << " = " << name << ";" << endl;
}

