dnl dpwe's aclocal.m4
dnl
dnl A place for my own autoconf macros
dnl 
dnl 1998jan23 Dan Ellis dpwe@icsi.berkeley.edu
dnl $Header: /n/yam/da/dpwe/share/lib/RCS/aclocal.m4,v 1.3 1998/01/26 22:28:04 dpwe Exp dpwe $
dnl

#--------------------------------------------------------------------
# ACDPWE_INIT_PREFIX
#	Make sure prefix and exec_prefix are not set to NONE
#	- anticipates standard end-of-config code so we can use 
#	prefix & exec_prefix earlier
#--------------------------------------------------------------------

AC_DEFUN(ACDPWE_INIT_PREFIX, [
  if test "x$prefix" = "xNONE" ; then
    prefix=$ac_default_prefix
  fi
  if test "x$exec_prefix" = "xNONE" ; then
    exec_prefix=$prefix
  fi
])

#--------------------------------------------------------------------
# ACDPWE_PROG_CC_CXX
#	Setup CC and CXX as:
#	   (a) gcc and g++ (by AC macros) *if* --enable-gcc specified
#	   (b) entry values of $CC and $CXX
#	   (c) "cc" and "CC"
#	Specify the first argument as "0" to omit the CXX checks
#--------------------------------------------------------------------

AC_DEFUN(ACDPWE_PROG_CC_CXX, [
  # Check the input argument; if $1="0", omit C++ checks.  If it's anything
  # else, or omitted, do C++ checks too.
  dpwe_doCxx=$1
  if test "x$dpwe_doCxx" = "x0" ; then
    dpwe_doCxx=""
  else
    dpwe_doCxx="1"
  fi
  AC_ARG_ENABLE(gcc, [  --enable-gcc            allow use of gcc if available],
    [dpwe_ok=$enableval], [dpwe_ok=no])
  if test "$dpwe_ok" = "yes"; then
      AC_PROG_CC
      test -n "$dpwe_doCxx" && \
        AC_PROG_CXX
  else
      CC=${CC-cc}
      test -n "$dpwe_doCxx" && \
        CXX=${CXX-CC}
  fi
  AC_MSG_CHECKING([default compiler flags])
  if test -z "$CFLAGS" ; then
      CFLAGS="-O"
  fi
  AC_ARG_WITH(cflags, [  --with-cflags=FLAGS     set compiler flags to FLAGS],
      [CFLAGS="$with_cflags"])
  AC_MSG_RESULT([$CFLAGS])

  AC_SUBST(CC)
  AC_SUBST(CFLAGS)
  if test -n "$dpwe_doCxx" ; then
    AC_MSG_CHECKING([default C++ compiler flags])
    if test -z "$CXXFLAGS" ; then
	CXXFLAGS="-O"
    fi
    AC_ARG_WITH(cxxflags, [  --with-cxxflags=FLAGS   set C++ compiler flags to FLAGS],
	[CXXFLAGS="$with_cxxflags"])
    AC_MSG_RESULT([$CXXFLAGS])
    AC_SUBST(CXX)
    AC_SUBST(CXXFLAGS)
  fi
])


#--------------------------------------------------------------------
# ACDPWE_LIB_MATH: (after tcl's config.in)
#	On a few very rare systems, all of the libm.a stuff is
#	already in libc.a.  Set compiler flags accordingly.
#	Also, Linux requires the "ieee" library for math to work
#	right (and it must appear before "-lm").
#--------------------------------------------------------------------

AC_DEFUN(ACDPWE_LIB_MATH, [
  AC_CHECK_FUNC(sin, MATH_LIBS="", MATH_LIBS="-lm")
  AC_CHECK_LIB(ieee, main, [MATH_LIBS="-lieee $MATH_LIBS"])
])

#--------------------------------------------------------------------
# ACDPWE_FLAGS_FULLSRCDIR
#	Setup variable $fullSrcDir to be an absolute path to the 
#	same directory as $srcdir (but don't expand ugly automounter 
#	paths if $srcdir is already absolute).
#--------------------------------------------------------------------

AC_DEFUN(ACDPWE_FLAGS_FULLSRCDIR, [
  if test "`echo $srcdir | sed -e 's/\(.\).*/\1/'`" = "/" ; then
    # srcdir is absolute - use as is
    fullSrcDir=$srcdir
  else
    # srcdir is a relative path - have to resolve it
    fullSrcDir=`cd $srcdir; pwd`
  fi
])

#--------------------------------------------------------------------
# ACDPWE_PKG_TCL
#	Assume dependency on tcl; attempt to read the tclConfig.sh
#	definitions, offering --with-tcl.
#	Sets up TCL_BIN_DIR, and whatever else is defined in tclConfig.sh.
#--------------------------------------------------------------------

AC_DEFUN(ACDPWE_PKG_TCL, [ 
  # Ensure exec_prefix is set...
  AC_REQUIRE([ACDPWE_INIT_PREFIX])
  # Default Tcl version to look for
  TCL_VERSION_GUESS=7.6
  AC_ARG_WITH(tcl, 
      [  --with-tcl=DIR          use Tcl $TCL_VERSION_GUESS binaries from DIR],
      TCL_BIN_DIR=$withval, TCL_BIN_DIR=none)
  if test "$TCL_BIN_DIR" = "none" ; then
    # Try to guess - look for a peer first
    TCL_BIN_DIR=$srcdir/../tcl$TCL_VERSION_GUESS/unix
    if test -d $TCL_BIN_DIR ; then
      # found it - resolve name
      TCL_BIN_DIR=`cd $TCL_BIN_DIR; pwd`
    else
      # did not find it - use best guess
      # but check for itcl on the off-chance
      if test -d $exec_prefix/lib/itcl ; then 
	TCL_BIN_DIR=$exec_prefix/lib/itcl
      else
	TCL_BIN_DIR=$exec_prefix/lib
      fi
    fi
  fi
  if test ! -d $TCL_BIN_DIR; then
    AC_MSG_ERROR(Tcl directory $TCL_BIN_DIR doesn't exist)
  fi
  if test ! -f $TCL_BIN_DIR/tclConfig.sh; then
    AC_MSG_ERROR(There's no tclConfig.sh in $TCL_BIN_DIR;  perhaps you didn't specify the Tcl *build* directory (not the toplevel Tcl directory) or you forgot to configure Tcl?)
  fi
  # Load the config file definitions
  dpwe_file=$TCL_BIN_DIR/tclConfig.sh
  . $dpwe_file
  if test -z "$TCL_LIB_RUNTIME_DIR" ; then
    # The "lib runtime dir" is where libtcl sits.  It's usually specified 
    #  as -L... in the first term of TCL_LIB_SPEC - try to pull it out...
    TCL_LIB_RUNTIME_DIR=`echo "$TCL_LIB_SPEC" | sed -e "s@ .*@@" -e "s@-L@@"`
    # This (one place) to find the tcl.h that you'll need to link against.
    # Actually, it gets installed in $prefix/include/itcl, but there's 
    # nothing pointing there, unfortunately
    TCL_INC_SPEC="-I$TCL_SRC_DIR/generic"
  fi
])

#--------------------------------------------------------------------
# ACDPWE_CHECK_TCL_SHLIB
#	Check that Tcl was configured for shared libraries, and set 
#	all the SHLIB* variables from the ones read from the tclConfig.sh 
#	file.
#--------------------------------------------------------------------

AC_DEFUN(ACDPWE_CHECK_TCL_SHLIB, [
  # Check that Tcl was configured & built for shared libraries
  AC_REQUIRE([ACDPWE_PKG_TCL])
  if test -z "$TCL_SHLIB_LD" ; then
    AC_MSG_ERROR(Tcl was not configured to use dynamic shared libraries)
  else
    # CFLAGS=$TCL_CFLAGS
    SHLIB_CFLAGS=$TCL_SHLIB_CFLAGS
    SHLIB_LD=$TCL_SHLIB_LD
    SHLIB_LD_LIBS=$TCL_SHLIB_LD_LIBS
    SHLIB_SUFFIX=$TCL_SHLIB_SUFFIX
    SHLIB_VERSION=$TCL_SHLIB_VERSION
    DL_LIBS=$TCL_DL_LIBS
    LD_FLAGS=$TCL_LD_FLAGS
    LD_SEARCH_FLAGS=$TCL_LD_SEARCH_FLAGS
    LIB_RUNTIME_DIR=$TCL_LIB_RUNTIME_DIR
  fi
])

#--------------------------------------------------------------------
# ACDPWE_FLAGS_SHLIB_CC_CXX
#	Figure out all the system-dependent stuff to configure the 
#	SHLIB* flags for compiling shared libraries (e.g. for Tcl 
#	dynamic loadable objects).  After tcl's configure.in
#	Re-uses the dpwe_doCxx flag set up in ACDPWE_PROG_CC_CXX
#--------------------------------------------------------------------

AC_DEFUN(ACDPWE_FLAGS_SHLIB_CC_CXX, [
  
  AC_REQUIRE([ACDPWE_PROG_CC_CXX])
  # Assume dpwe_doCxx has already been set up.

  #--------------------------------------------------------------------
  #	The statements below define a collection of symbols related to
  #	dynamic loading and shared libraries:
  #
  #	LD_FLAGS -	Flags to pass to the compiler when linking object
  #			files into an executable application binary such
  #			as tclsh.
  #	LD_SEARCH_FLAGS-Flags to pass to ld, such as "-R /usr/local/tcl/lib",
  #			that tell the run-time dynamic linker where to look
  #			for shared libraries such as libtcl.so.  Depends on
  #			the variable LIB_RUNTIME_DIR in the Makefile.
  #	SHLIB_CFLAGS -	Flags to pass to cc when compiling the components
  #			of a shared library (may request position-independent
  #			code, among other things).
  #	SHLIB_LD -	Base command to use for combining object files
  #			into a shared library.
  #	SHLIB_LD_LIBS -	Dependent libraries for the linker to scan when
  #			creating shared libraries.  This symbol typically
  #			goes at the end of the "ld" commands that build
  #			shared libraries. The value of the symbol is
  #			"${LIBS}" if all of the dependent libraries should
  #			be specified when creating a shared library.  If
  #			dependent libraries should not be specified (as on
  #			SunOS 4.x, where they cause the link to fail, or in
  #			general if Tcl and Tk aren't themselves shared
  #			libraries), then this symbol has an empty string
  #			as its value.
  #	SHLIB_SUFFIX -	Suffix to use for the names of dynamically loadable
  #			extensions.  An empty string means we don't know how
  #			to use shared libraries on this platform.
  # Also: if doing CXX, SHLIB_CXXFLAGS SHLIB_LDXX SHLIB_LDXX_LIBS SHLD_RANLIB
  #--------------------------------------------------------------------

  # Step 1: set the variable "system" to hold the name and version number
  # for the system.  This can usually be done via the "uname" command, but
  # there are a few systems, like Next, where this doesn't work.

  AC_MSG_CHECKING([system version (for dynamic loading)])
  if test -f /usr/lib/NextStep/software_version; then
      system=NEXTSTEP-`awk '/3/,/3/' /usr/lib/NextStep/software_version`
  else
      system=`uname -s`-`uname -r`
      if test "$?" -ne 0 ; then
	  AC_MSG_RESULT([unknown (can't find uname command)])
	  system=unknown
      else
	  # Special check for weird MP-RAS system (uname returns weird
	  # results, and the version is kept in special file).

	  if test -r /etc/.relid -a "X`uname -n`" = "X`uname -s`" ; then
	      system=MP-RAS-`awk '{print $3}' /etc/.relid'`
	  fi
	  if test "`uname -s`" = "AIX" ; then
	      system=AIX-`uname -v`.`uname -r`
	  fi
	  AC_MSG_RESULT($system)
      fi
  fi

  # Step 2: check for existence of -ldl library.  This is needed because
  # Linux can use either -ldl or -ldld for dynamic loading.

  AC_CHECK_LIB(dl, dlopen, have_dl=yes, have_dl=no)

  # Step 3: set configuration options based on system name and version.

  AC_CANONICAL_SYSTEM

  fullSrcDir=`cd $srcdir; pwd`
  AIX=no
  SHARED_LIB_SUFFIX=""
  UNSHARED_LIB_SUFFIX=""
  LIB_VERSIONS_OK=ok
  case $system in
      AIX-*)
	  SHLIB_CFLAGS=""
	  SHLIB_LD="$fullSrcDir/ldAix /bin/ld -bhalt:4 -bM:SRE -bE:lib.exp -H512 -T512"
	  SHLIB_LD_LIBS='${LIBS}'
	  SHLIB_SUFFIX=".so"
	  LD_FLAGS=""
	  LD_SEARCH_FLAGS='-L${LIB_RUNTIME_DIR}'
	  AIX=yes
	  ;;
      BSD/OS-2.1*)
	  SHLIB_CFLAGS=""
	  SHLIB_LD="ld -r"
	  SHLIB_LD_FLAGS=""
	  SHLIB_SUFFIX=".so"
	  LD_FLAGS=""
	  LD_SEARCH_FLAGS=""
	  ;;
      HP-UX-*.08.*|HP-UX-*.09.*|HP-UX-*.10.*)
	  AC_CHECK_LIB(dld, shl_load, tcl_ok=yes, tcl_ok=no)
	  if test "$tcl_ok" = yes; then
	      SHLIB_CFLAGS="+z"
	      SHLIB_LD="ld -b"
	      SHLIB_LD_LIBS='${LIBS}'
	      SHLIB_SUFFIX=".sl"
	      LD_FLAGS="-Wl,-E"
	      LD_SEARCH_FLAGS='-Wl,+b,${LIB_RUNTIME_DIR}:.'
	  fi
	  ;;
      IRIX-4.*)
	  SHLIB_CFLAGS="-G 0"
	  SHLIB_SUFFIX="..o"
	  SHLIB_LD="echo tclLdAout $CC \{$SHLIB_CFLAGS\} | `pwd`/tclsh -r -G 0"
	  SHLIB_LD_LIBS='${LIBS}'
	  LD_FLAGS="-Wl,-D,08000000"
	  LD_SEARCH_FLAGS=""
	  ;;
      IRIX-5.*|IRIX-6.*)
	  SHLIB_CFLAGS=""
	  SHLIB_LD="ld -shared -rdata_shared -update_registry /usr/tmp/so_locations"
	  SHLIB_LD_LIBS=""
	  SHLIB_SUFFIX=".so"
	  SHLD_RANLIB='${RANLIB}'
	  LD_FLAGS=""
	  LD_SEARCH_FLAGS='-Wl,-rpath,${LIB_RUNTIME_DIR}'
          if test -n "$dpwe_doCxx" ; then
	    SHLIB_CXXFLAGS=""
	    SHLIB_LDXX="$(CXX) -shared -rdata_shared -update_registry /usr/tmp/so_locations"
	    SHLIB_LDXX_LIBS='${LIBS}'
          fi
	  ;;
      IRIX64-6.*)
	  SHLIB_CFLAGS=""
	  SHLIB_LD="ld -32 -shared -rdata_shared -rpath /usr/local/lib"
	  SHLIB_LD_LIBS=""
	  SHLIB_SUFFIX=".so"
	  LD_FLAGS=""
	  LD_SEARCH_FLAGS='-Wl,-rpath,${LIB_RUNTIME_DIR}'
	  ;;
      Linux*)
	  SHLIB_CFLAGS="-fPIC"
	  SHLIB_LD_LIBS=""
	  SHLIB_SUFFIX=".so"
	  if test "$have_dl" = yes; then
	      SHLIB_LD="${CC} -shared"
	      LD_FLAGS="-rdynamic"
	      LD_SEARCH_FLAGS=""
	  else
	      AC_CHECK_HEADER(dld.h, [
		  SHLIB_LD="ld -shared"
		  LD_FLAGS=""
		  LD_SEARCH_FLAGS=""])
	  fi
	  SHLD_RANLIB='${RANLIB}'
          if test -n "$dpwe_doCxx" ; then
	    SHLIB_CXXFLAGS="-fPIC"
	    SHLIB_LDXX="${CXX} -shared"
	    SHLIB_LDXX_LIBS=""
	  fi
	  ;;
      MP-RAS-02*)
	  SHLIB_CFLAGS="-K PIC"
	  SHLIB_LD="cc -G"
	  SHLIB_LD_LIBS=""
	  SHLIB_SUFFIX=".so"
	  LD_FLAGS=""
	  LD_SEARCH_FLAGS=""
	  ;;
      MP-RAS-*)
	  SHLIB_CFLAGS="-K PIC"
	  SHLIB_LD="cc -G"
	  SHLIB_LD_LIBS=""
	  SHLIB_SUFFIX=".so"
	  LD_FLAGS="-Wl,-Bexport"
	  LD_SEARCH_FLAGS=""
	  ;;
      NetBSD-*|FreeBSD-*)
	  # Not available on all versions:  check for include file.
	  AC_CHECK_HEADER(dlfcn.h, [
	      SHLIB_CFLAGS="-fpic"
	      SHLIB_LD="ld -Bshareable -x"
	      SHLIB_LD_LIBS=""
	      SHLIB_SUFFIX=".so"
	      LD_FLAGS=""
	      LD_SEARCH_FLAGS=""
	  ], [
	      SHLIB_CFLAGS=""
	      SHLIB_LD="echo tclLdAout $CC \{$SHLIB_CFLAGS\} | `pwd`/tclsh -r -G 0"
	      SHLIB_LD_LIBS='${LIBS}'
	      SHLIB_SUFFIX="..o"
	      LD_FLAGS=""
	      LD_SEARCH_FLAGS=""
	  ])
	  # FreeBSD doesn't handle version numbers with dots.  Also, have to
	  # append a dummy version number to .so file names.
	  ;;
      NEXTSTEP-*)
	  SHLIB_CFLAGS=""
	  SHLIB_LD="cc -nostdlib -r"
	  SHLIB_LD_LIBS=""
	  SHLIB_SUFFIX=".so"
	  LD_FLAGS=""
	  LD_SEARCH_FLAGS=""
	  ;;
      OSF1-1.0|OSF1-1.1|OSF1-1.2)
	  # OSF/1 1.[012] from OSF, and derivatives, including Paragon OSF/1
	  SHLIB_CFLAGS=""
	  # Hack: make package name same as library name
	  SHLIB_LD='ld -R -export $@:'
	  SHLIB_LD_LIBS=""
	  SHLIB_SUFFIX=".so"
	  LD_FLAGS=""
	  LD_SEARCH_FLAGS=""
	  ;;
      OSF1-1.*)
	  # OSF/1 1.3 from OSF using ELF, and derivatives, including AD2
	  SHLIB_CFLAGS="-fpic"
	  SHLIB_LD="ld -shared"
	  SHLIB_LD_LIBS=""
	  SHLIB_SUFFIX=".so"
	  LD_FLAGS=""
	  LD_SEARCH_FLAGS=""
	  ;;
      OSF1-V*)
	  # Digital OSF/1
	  SHLIB_CFLAGS=""
	  SHLIB_LD='ld -shared -expect_unresolved "*"'
	  SHLIB_LD_LIBS=""
	  SHLIB_SUFFIX=".so"
	  LD_FLAGS=""
	  LD_SEARCH_FLAGS='-Wl,-rpath,${LIB_RUNTIME_DIR}'
	  SHLD_RANLIB=":"
          if test -n "$dpwe_doCxx" ; then
	    SHLIB_CXXFLAGS=""
	    SHLIB_LDXX='ld -shared -expect_unresolved "*"'
	    SHLIB_LDXX_LIBS=""
	  fi
	  ;;
      SCO_SV-3.2*)
	  # Note, dlopen is available only on SCO 3.2.5 and greater.  However,
	  # this test works, since "uname -s" was non-standard in 3.2.4 and
	  # below.
	  SHLIB_CFLAGS="-Kpic -belf"
	  SHLIB_LD="ld -G"
	  SHLIB_LD_LIBS=""
	  SHLIB_SUFFIX=".so"
	  LD_FLAGS="-belf -Wl,-Bexport"
	  LD_SEARCH_FLAGS=""
	  ;;
       SINIX*5.4*)
	  SHLIB_CFLAGS="-K PIC"
	  SHLIB_LD="cc -G"
	  SHLIB_LD_LIBS=""
	  SHLIB_SUFFIX=".so"
	  LD_FLAGS=""
	  LD_SEARCH_FLAGS=""
	  ;;
      SunOS-4*)
	  SHLIB_CFLAGS="-PIC"
	  SHLIB_LD="ld -assert pure-text"
	  SHLIB_LD_LIBS=""
	  SHLIB_SUFFIX=".so"
	  LD_FLAGS=""
	  LD_SEARCH_FLAGS='-L${LIB_RUNTIME_DIR}'
	  # SunOS can't handle version numbers with dots in them in library
	  # specs, like -ltcl7.5, so use -ltcl75 instead.  Also, it
	  # requires an extra version number at the end of .so file names.
	  # So, the library has to have a name like libtcl75.so.1.0
	  SHLD_RANLIB=":"
          if test -n "$dpwe_doCxx" ; then
	    SHLIB_CXXFLAGS="-pic"
	    # SHLIB_LDXX="ld -assert pure-text"
	    # SHLIB_LDXX_LIBS=""
	    SHLIB_LDXX="ld"
	    SHLIB_LDXX_LIBS=""
	  fi
	  ;;
      SunOS-5*)
	  SHLIB_CFLAGS="-KPIC"
	  SHLIB_LD="/usr/ccs/bin/ld -G -z text"
	  SHLIB_LD_LIBS='${LIBS}'
	  SHLIB_SUFFIX=".so"
	  LD_FLAGS=""
	  LD_SEARCH_FLAGS='-R ${LIB_RUNTIME_DIR}'
	  SHLD_RANLIB='${RANLIB}'
          if test -n "$dpwe_doCxx" ; then
	    SHLIB_CXXFLAGS="-pic"
	    SHLIB_LDXX="$CXX -G -z text"
	    SHLIB_LDXX_LIBS='${LIBS}'
	  fi
	  ;;
      ULTRIX-4.*)
	  SHLIB_CFLAGS="-G 0"
	  SHLIB_SUFFIX="..o"
	  SHLIB_LD="echo tclLdAout $CC \{$SHLIB_CFLAGS\} | `pwd`/tclsh -r -G 0"
	  SHLIB_LD_LIBS='${LIBS}'
	  LD_FLAGS="-Wl,-D,08000000"
	  LD_SEARCH_FLAGS=""
	  ;;
      UNIX_SV*)
	  SHLIB_CFLAGS="-KPIC"
	  SHLIB_LD="cc -G"
	  SHLIB_LD_LIBS=""
	  SHLIB_SUFFIX=".so"
	  # Some UNIX_SV* systems (unixware 1.1.2 for example) have linkers
	  # that don't grok the -Bexport option.  Test that it does.
	  hold_ldflags=$LDFLAGS
	  AC_MSG_CHECKING(for ld accepts -Bexport flag)
	  LDFLAGS="${LDFLAGS} -Wl,-Bexport"
	  AC_TRY_LINK(, [int i;], found=yes, found=no)
	  LDFLAGS=$hold_ldflags
	  AC_MSG_RESULT($found)
	  if test $found = yes; then
	    LD_FLAGS="-Wl,-Bexport"
	  else
	    LD_FLAGS=""
	  fi
	  LD_SEARCH_FLAGS=""
	  ;;
  esac

  if test -z "$SHLIB_LD" ; then
    AC_MSG_ERROR(Cannot figure out how to compile shared libraries)
  fi

  if test -n "$dpwe_doCxx" && test -z "$SHLIB_LDXX" ; then
    AC_MSG_ERROR(C++ shared library building not yet configured for this arch)
  fi

  # If we're running gcc, then change the C flags for compiling shared
  # libraries to the right flags for gcc, instead of those for the
  # standard manufacturer compiler.

  if test "$CC" = "gcc" -o `$CC -v 2>&1 | grep -c gcc` != "0" ; then
    SHLIB_CFLAGS="-fPIC"
  fi

  if test -n "$dpwe_doCxx" ; then
    if test "$CXX" = "g++" -o "$CXX" = "gcc" -o `$CXX -v 2>&1 | grep -c gcc` != "0" ; then
	SHLIB_CXXFLAGS="-fPIC"
	SHLIB_LDXX=$SHLIB_LD
	SHLD_RANLIB=":"
    fi
  fi

])

#--------------------------------------------------------------------
# ACDPWE_CXX_SHLIB_STATIC_INIT
#	Test whether shared/dynamic libraries containing C++ statically-
#	initialized data actually gets statically initialized (I'm 
#	unable to make it do so under g++ 2.7)
#--------------------------------------------------------------------

AC_DEFUN(ACDPWE_CXX_SHLIB_STATIC_INIT, [
  # Make sure that we've configured for compiling C++ shared libraries
  AC_REQUIRE([ACDPWE_FLAGS_SHLIB_CC_CXX])
  AC_MSG_CHECKING(C++ static initializers in shared libraries)
  # Construct a test case
  tmproot="_acdpwe_tmp1"
  tmpsrc=${tmproot}.cc
  tmpobj=${tmproot}.o
  tmplib=${tmproot}$SHLIB_SUFFIX
  rm -f $tmpsrc
  cat > $tmpsrc << EOF
class A {
public:
  int i;
  A(void) { i = 1;}
};
static A a;
int test(void) {return a.i;}
EOF
  # Make it into a shared library
  $CXX $CXXFLAGS $SHLIB_CXXFLAGS -c $tmpsrc -o $tmpobj
  $SHLIB_LDXX -o $tmplib $tmpobj `eval echo $SHLIB_LDXX_LIBS`
  `eval echo $SHLD_RANLIB` $tmplib
  # Try running a program against it
  tmproot2="_acdpwe_tmp2"
  tmpsrc2=${tmproot2}.cc
  tmpobj2=${tmproot2}.o
  tmpprog=./${tmproot2}
  rm -f $tmpsrc2
  cat > $tmpsrc2 << EOF
extern int test(void);
int main(void) {
  int i = test();
  return i;
}
EOF
  # Build the executable
  $CXX $CXXFLAGS $SHLIB_CXXFLAGS -c $tmpsrc2 -o $tmpobj2
  LIB_RUNTIME_tmp=$LIB_RUNTIME_DIR
  LIB_RUNTIME_DIR=.
  $CXX $LDFLAGS $tmpobj2 $tmplib -o $tmpprog `eval echo $LD_SEARCH_FLAGS`
  if $tmpprog ; then
    # 'success' indicates zero exit code i.e. init was not called
    AC_MSG_RESULT(are not called)
    AC_DEFINE(SHLIB_NO_STATIC_INIT)
  else
    AC_MSG_RESULT(are called)
    AC_DEFINE(SHLIB_STATIC_INIT)
  fi
  # Clean up
  LIB_RUNTIME_DIR=$LIB_RUNTIME_tmp
  rm -f $tmpsrc $tmpobj $tmplib $tmpsrc2 $tmpobj2 $tmpprog
])
